@@ -6,7 +6,7 @@
 ;; Maintainer: Florian Rommel <mail@florommel.de>
 ;; Url: https://github.com/florommel/bufferlo
 ;; Created: 2021-09-15
-;; Version: 0.6
+;; Version: 0.7
 ;; Package-Requires: ((emacs "27.1"))
 ;; Keywords: buffer frame tabs local
 
@@ -33,30 +33,23 @@
 ;; including undeletion of frame and tabs, tab duplication and moving,
 ;; frame cloning, and persisting sessions (via desktop.el).
 
-;; A buffer is added to the local buffer list when it is displayed in
-;; the frame/tab (e.g., by opening a new file in the tab or by
-;; switching to the buffer from the global buffer list).  In addition,
-;; bufferlo provides functions that allow the manipulation of the
-;; local buffer list.  Bufferlo does not touch the global buffer list
-;; or the existing buffer-management facilities.  Use the equivalent
-;; bufferlo variants to work with the frame/tab local buffer list.
-
-;; The packages frame-bufs (unmaintained) and beframe provide similar
-;; functionality, but only at the frame level, without support for
-;; tabs and desktop.el.
-;; https://github.com/alpaker/frame-bufs
-;; https://protesilaos.com/emacs/beframe
-
-;; You may also have a look at full workspace solutions like bufler
-;; (automatic rule-based workspace management and buffer grouping) or
-;; perspective (comprehensive workspace isolation, workspace merging,
-;; workspace persistence).  They work quite differently than bufferlo.
-;; https://github.com/alphapapa/bufler.el
-;; https://github.com/nex3/perspective-el
+;; With bufferlo, every frame or tab (if you use tab-bar tabs) has an
+;; additional manageable local buffer list.  A buffer is added to the
+;; local buffer list when displayed in the frame/tab (e.g., by opening
+;; a new file in the tab or by switching to the buffer from the global
+;; buffer list).  Bufferlo provides extensive management functions for
+;; the local list and frame/tab-local variants of the switch-buffer
+;; function, buffer menu, and Ibuffer.  In addition, you can configure
+;; any command that selects a buffer to use the local buffer list
+;; (bufferlo anyhwere).  Bufferlo also allows you to bookmark and
+;; persist the state of individual frames or tabs.
 
 ;;; Code:
 
+(require 'seq)
+(require 'tab-bar)
 (require 'desktop)
+(require 'bookmark)
 
 (defgroup bufferlo nil
   "Manage frame/tab-local buffer lists."
@@ -134,7 +127,10 @@ You can set this to \"*scratch*\"."
 If nil, the local scratch buffers' major mode is set to `initial-major-mode'."
   :type 'function)
 
-(defcustom bufferlo-anywhere-filter '(switch-to-buffer bufferlo-switch-to-buffer)
+(defcustom bufferlo-anywhere-filter '(switch-to-buffer
+                                      bufferlo-switch-to-buffer
+                                      bufferlo-find-buffer
+                                      bufferlo-find-buffer-switch)
   "The functions that use the local buffer list in `bufferlo-anywhere-mode'.
 If `bufferlo-anywhere-filter-type' is set to `exclude', this is an exclude
 filter (i.e., determines the functions that do not use the local buffer list).
@@ -154,6 +150,21 @@ Set this to `include' or `exclude'."
   :type '(radio (const :tag "Include filter" include)
                 (const :tag "Exclude filter" exclude)))
 
+(defcustom bufferlo-bookmark-map-functions nil
+  "Functions to call for every local buffer when making a tab bookmark.
+Each function takes a bookmark record as its argument.  The corresponding
+buffer is set as current buffer.  Every function should return a valid
+bookmark record or nil.  The first function gets the buffer's default
+bookmark record or nil if it is not bookmarkable.  Subsequent functions
+receive the bookmark record that the previous function returned as their
+argument.  The bookmark record of the last function is used as the
+effective record.  If the last function returns nil, no record for the
+respective buffer is included in the frame or tab bookmark.
+
+These functions are also called when creating a frame bookmark, since a
+frame bookmark is a collection of tab bookmarks."
+  :type 'hook)
+
 (defvar bufferlo--desktop-advice-active nil)
 (defvar bufferlo--desktop-advice-active-force nil)
 
@@ -258,10 +269,10 @@ before calling OLDFN with ARGS.  See `bufferlo--clear-buffer-lists'."
     ;; This workaround selects a buffer that is in the local list in such a case.
     (unless (bufferlo-local-buffer-p (current-buffer) nil nil t)
       (let ((buffer (or
-                     (cl-find-if-not #'minibufferp
-                                     (frame-parameter nil 'buffer-list))
-                     (cl-find-if-not #'minibufferp
-                                     (frame-parameter nil 'buried-buffer-list)))))
+                     (seq-find (lambda (b) (not (minibufferp b)))
+                               (frame-parameter nil 'buffer-list))
+                     (seq-find (lambda (b) (not (minibufferp b)))
+                               (frame-parameter nil 'buried-buffer-list)))))
         (switch-to-buffer buffer t t)))
 
     result))
@@ -412,10 +423,20 @@ the adviced functions."
         (bufferlo--desktop-advice-active-force t))
     (apply oldfn args)))
 
+(defsubst bufferlo--warn ()
+  "Warn if `bufferlo-mode' is not enabled."
+  (defvar bufferlo--warn-current-command nil)
+  (when (and (not bufferlo-mode)
+             (not (eq this-command bufferlo--warn-current-command)))
+    (setq bufferlo--warn-current-command this-command)
+    (message (format "%s: bufferlo-mode should be enabled"
+                     this-command))))
+
 (defun bufferlo-clear (&optional frame)
   "Clear the frame/tab's buffer list, except for the current buffer.
 If FRAME is nil, use the current frame."
   (interactive)
+  (bufferlo--warn)
   (set-frame-parameter frame 'buffer-list
                        (list (if frame
                                  (with-selected-frame frame
@@ -431,6 +452,7 @@ If FRAME is nil, use the current frame."
                        (bufferlo-buffer-list))))
       (read-buffer "Remove buffer: " nil t
                    (lambda (b) (member (car b) lbs))))))
+  (bufferlo--warn)
   (let ((bl (frame-parameter nil 'buffer-list))
         (bbl (frame-parameter nil 'buried-buffer-list))
         (buffer (get-buffer buffer)))
@@ -452,6 +474,7 @@ If FRAME is nil, use the current frame."
 (defun bufferlo-remove-non-exclusive-buffers ()
   "Remove all buffers from the local buffer list that are not exclusive to it."
   (interactive)
+  (bufferlo--warn)
   (dolist (buffer (bufferlo--get-exclusive-buffers nil t))
     (bufferlo-remove buffer)))
 
@@ -460,6 +483,7 @@ If FRAME is nil, use the current frame."
 If `bufferlo-include-buried-buffers' is set to nil then this has the same
 effect as a simple `bury-buffer'."
   (interactive)
+  (bufferlo--warn)
   (let ((buffer (window-normalize-buffer buffer-or-name)))
     (bury-buffer-internal buffer)
     (bufferlo-remove buffer)))
@@ -512,6 +536,7 @@ local lists of other frames and tabs are killed too.
 Buffers matching `bufferlo-kill-buffers-exclude-filters' are never killed.
 If FRAME is nil, use the current frame."
   (interactive "P")
+  (bufferlo--warn)
   (let ((exclude (bufferlo--merge-regexp-list
                   (append '("a^") bufferlo-kill-buffers-exclude-filters)))
         (kill-list (if killall
@@ -525,6 +550,7 @@ If FRAME is nil, use the current frame."
   "Kill all buffers that are not in any local list of a frame or tab.
 Buffers matching `bufferlo-kill-buffers-exclude-filters' are never killed."
   (interactive)
+  (bufferlo--warn)
   (let ((exclude (bufferlo--merge-regexp-list
                   (append '("a^") bufferlo-kill-buffers-exclude-filters))))
     (dolist (buffer (bufferlo--get-orphan-buffers))
@@ -535,6 +561,7 @@ Buffers matching `bufferlo-kill-buffers-exclude-filters' are never killed."
   "Delete a frame and kill the local buffers.
 If FRAME is nil, kill the current frame."
   (interactive)
+  (bufferlo--warn)
   (bufferlo-kill-buffers frame)
   (delete-frame))
 
@@ -542,6 +569,7 @@ If FRAME is nil, kill the current frame."
   "Close the current tab and kill the local buffers.
 The optional parameter KILLALL is passed to `bufferlo-kill-buffers'"
   (interactive "P")
+  (bufferlo--warn)
   (bufferlo-kill-buffers killall)
   (tab-bar-close-tab))
 
@@ -552,6 +580,7 @@ the local buffer list.  When FILE-BUFFERS-ONLY is non-nil or the
 prefix argument is given, remove only buffers that visit a file.
 Buffers matching `bufferlo-include-buffer-filters' are not removed."
   (interactive "P")
+  (bufferlo--warn)
   (let ((curr-project (project-current))
         (include (bufferlo--merge-regexp-list
                   (append '("a^") bufferlo-include-buffer-filters))))
@@ -570,6 +599,7 @@ If multiple frame or tabs contain the buffer, interactively prompt
 for the to-be-selected frame and tab.
 This does not select the buffer -- just the containing frame and tab."
   (interactive "b")
+  (bufferlo--warn)
   (let* ((buffer (get-buffer buffer-or-name))
          (flatten (lambda (list)
                     (apply #'append (append list '()))))
@@ -635,6 +665,7 @@ This does not select the buffer -- just the containing frame and tab."
   "Switch to the frame/tab containig BUFFER-OR-NAME and select the buffer.
 This is like `bufferlo-find-buffer' but additionally selects the buffer."
   (interactive "b")
+  (bufferlo--warn)
   (when (bufferlo-find-buffer buffer-or-name)
     (switch-to-buffer buffer-or-name)))
 
@@ -705,6 +736,7 @@ If the prefix arument is given, include all buffers."
         (read-buffer
          "Switch to local buffer: " (other-buffer (current-buffer)) nil
          (lambda (b) (member (if (stringp b) b (car b)) lbs)))))))
+  (bufferlo--warn)
   (switch-to-buffer buffer norecord force-same-window))
 
 (defvar-local bufferlo--buffer-menu-this-frame nil)
@@ -716,6 +748,7 @@ If the prefix arument is given, include all buffers."
 (defun bufferlo-list-buffers ()
   "Display a list of local buffers."
   (interactive)
+  (bufferlo--warn)
   (display-buffer
    (let* ((old-buffer (current-buffer))
           (name (or
@@ -740,6 +773,7 @@ If the prefix arument is given, include all buffers."
 (defun bufferlo-list-orphan-buffers ()
   "Display a list of orphan buffers."
   (interactive)
+  (bufferlo--warn)
   (display-buffer
    (let* ((old-buffer (current-buffer))
           (name "*Orphan Buffer List*")
@@ -778,6 +812,7 @@ If the prefix arument is given, include all buffers."
 Every frame/tab gets its own local bufferlo ibuffer buffer.
 The parameters OTHER-WINDOW-P NOSELECT SHRINK are passed to `ibuffer'."
   (interactive)
+  (bufferlo--warn)
   (let ((name (or
                (seq-find (lambda (b)
                            (string-match-p
@@ -792,6 +827,7 @@ The parameters OTHER-WINDOW-P NOSELECT SHRINK are passed to `ibuffer'."
   "Invoke `ibuffer' filtered for orphan buffers.
 The parameters OTHER-WINDOW-P NOSELECT SHRINK are passed to `ibuffer'."
   (interactive)
+  (bufferlo--warn)
   (let ((name "*Bufferlo Orphans Ibuffer*"))
     (ibuffer other-window-p name '((bufferlo-orphan-buffers . nil))
              noselect shrink)))
@@ -811,7 +847,10 @@ the next command, when the mode is enabled."
   :lighter nil
   :keymap nil
   (if bufferlo-anywhere-mode
-      (advice-add #'call-interactively :around #'bufferlo--interactive-advice)
+      (progn
+        (bufferlo--warn)
+        (advice-add #'call-interactively
+                    :around #'bufferlo--interactive-advice))
     (advice-remove #'call-interactively #'bufferlo--interactive-advice)))
 
 (defvar bufferlo--anywhere-tmp-enabled nil)
@@ -910,6 +949,272 @@ In contrast to `bufferlo-anywhere-mode', this does not adhere to
       (advice-add #'call-interactively :around #'bufferlo--interactive-advice))
     (add-hook 'post-command-hook postfun)))
 
+(defun bufferlo--bookmark-get-for-buffer (buffer)
+  "Get `buffer-name' and bookmark for BUFFER."
+  (with-current-buffer buffer
+    (let ((record (when (or (and (eq bookmark-make-record-function
+                                     #'bookmark-make-record-default)
+                                 (ignore-errors (bookmark-buffer-file-name)))
+                            (not (eq bookmark-make-record-function
+                                     #'bookmark-make-record-default)))
+                    (bookmark-make-record))))
+      (dolist (fn bufferlo-bookmark-map-functions)
+        (setq record (funcall fn record)))
+      (list (buffer-name buffer) record))))
+
+(defun bufferlo--bookmark-get-for-buffers-in-tab (frame)
+  "Get bookmarks for all buffers of the tab TABNUM in FRAME."
+  (with-selected-frame (or frame (selected-frame))
+    (seq-filter #'identity
+                (mapcar #'bufferlo--bookmark-get-for-buffer
+                        (bufferlo-buffer-list frame nil t)))))
+
+(defun bufferlo--bookmark-tab-get (&optional name frame)
+  "Get the bufferlo tab bookmark for the current tab in FRAME.
+Optional argument NAME provides a name for the bookmarks.
+FRAME specifies the frame; the default value of nil selects the current frame."
+  `((buffer-bookmarks . ,(bufferlo--bookmark-get-for-buffers-in-tab frame))
+    (buffer-list . ,(mapcar #'buffer-name (bufferlo-buffer-list frame nil t)))
+    (window . ,(window-state-get (frame-root-window frame) 'writable))
+    (name . ,name)
+    (handler . ,#'bufferlo--bookmark-tab-handler)))
+
+(defun bufferlo--ws-replace-buffer-names (ws replace-alist)
+  "Replace buffer names according to REPLACE-ALIST in the window state WS."
+  (dolist (el ws)
+    (when-let (type (and (listp el) (car el)))
+      (cond ((memq type '(vc hc))
+             (bufferlo--ws-replace-buffer-names (cdr el) replace-alist))
+            ((eq type 'leaf)
+             (let ((bc (assq 'buffer (cdr el))))
+               (when-let (replace (assoc (cadr bc) replace-alist))
+                 (setf (cadr bc) (cdr replace)))))))))
+
+(defun bufferlo--bookmark-tab-handler (bookmark &optional no-message)
+  "Handle bufferlo tab bookmark.
+The argument BOOKMARK is the to-be restored tab bookmark created via
+`bufferlo--bookmark-tab-get'.  The optional argument NO-MESSAGE inhibits
+the message after successfully restoring the bookmark."
+  (let* ((ws (copy-tree (alist-get 'window bookmark)))
+         (dummy (generate-new-buffer " *bufferlo dummy buffer*"))
+         (renamed
+          (mapcar
+           (lambda (bm)
+             (let ((org-name (car bm))
+                   (record (cadr bm)))
+               (set-buffer dummy)
+               (condition-case err
+                   (progn (funcall (or (bookmark-get-handler record)
+                                       'bookmark-default-handler)
+                                   record)
+                          (run-hooks 'bookmark-after-jump-hook))
+                 (error
+                  (ignore err)
+                  (message "Bufferlo tab: Could not restore %s" org-name)))
+               (unless (eq (current-buffer) dummy)
+                 (unless (string-equal org-name (buffer-name))
+                   (cons org-name (buffer-name))))))
+           (alist-get 'buffer-bookmarks bookmark)))
+         (bl (mapcar (lambda (b)
+                       (if-let (replace (assoc b renamed))
+                           (cdr replace)
+                         b))
+                     (alist-get 'buffer-list bookmark)))
+         (bl (seq-filter #'get-buffer bl))
+         (bl (mapcar #'get-buffer bl)))
+    (kill-buffer dummy)
+    (bufferlo--ws-replace-buffer-names ws renamed)
+    (window-state-put ws (frame-root-window))
+    (set-frame-parameter nil 'buffer-list bl)
+    (set-frame-parameter nil 'buried-buffer-list nil))
+  (unless no-message
+    (message "Restored bufferlo tab bookmark%s"
+             (if-let (name (alist-get 'name bookmark))
+                 (format ": %s" name) ""))))
+
+(defun bufferlo--bookmark-frame-get (&optional name frame)
+  "Get the bufferlo frame bookmark.
+Optional argument NAME provides a name for the bookmarks.
+FRAME specifies the frame; the default value of nil selects the current frame."
+  (let ((org-tab (1+ (tab-bar--current-tab-index nil frame)))
+        (tabs nil))
+    (dotimes (i (length (funcall tab-bar-tabs-function)))
+      (tab-bar-select-tab (1+ i))
+      (let* ((curr (alist-get 'current-tab (funcall tab-bar-tabs-function)))
+             (name (alist-get 'name curr))
+             (explicit-name (alist-get 'explicit-name curr))
+             (tbm (bufferlo--bookmark-tab-get nil frame)))
+        (if explicit-name
+            (push (cons 'tab-name name) tbm)
+          (push (cons 'tab-name nil) tbm))
+        (push tbm tabs)))
+    (tab-bar-select-tab org-tab)
+    `((tabs . ,(reverse tabs))
+      (current . ,org-tab)
+      (name . ,name)
+      (handler . ,#'bufferlo--bookmark-frame-handler))))
+
+(defun bufferlo--bookmark-frame-handler (bookmark &optional no-message)
+  "Handle bufferlo frame bookmark.
+The argument BOOKMARK is the to-be restored frame bookmark created via
+`bufferlo--bookmark-frame-get'.  The optional argument NO-MESSAGE inhibits
+the message after successfully restoring the bookmark."
+  (if (>= emacs-major-version 28)
+      (tab-bar-tabs-set nil)
+    (set-frame-parameter nil 'tabs nil))
+  (let ((first t))
+    (mapc
+     (lambda (tbm)
+       (if first
+           (setq first nil)
+         (tab-bar-new-tab-to))
+       (bufferlo--bookmark-tab-handler tbm t)
+       (when-let (tab-name (alist-get 'tab-name tbm))
+         (tab-bar-rename-tab tab-name)))
+     (alist-get 'tabs bookmark)))
+  (tab-bar-select-tab (alist-get 'current bookmark))
+  (unless no-message
+    (message "Restored bufferlo frame bookmark%s"
+             (if-let (name (alist-get 'name bookmark))
+                 (format ": %s" name) ""))))
+
+(defun bufferlo--bookmark-get-names (&rest handlers)
+  "Get the names of all existing bookmarks for HANDLERS."
+  (bookmark-maybe-load-default-file)
+  (mapcar
+   #'car
+   (seq-filter
+    (lambda (bm)
+      (memq (alist-get 'handler (cdr bm)) handlers))
+    bookmark-alist)))
+
+(defun bufferlo--current-tab ()
+  "Get the current tab record."
+  (if (>= emacs-major-version 28)
+      (tab-bar--current-tab-find)
+    (assq 'current-tab (funcall tab-bar-tabs-function nil))))
+
+(defun bufferlo-bookmark-tab-save (name &optional no-overwrite)
+  "Save the current tab as a bookmark.
+NAME is the bookmark's name.  If NO-OVERWRITE is non-nil,
+record the new bookmark without throwing away the old one.
+
+This function persists the current tab's state:
+The resulting bookmark stores the window configuration and the local
+buffer list of the current tab.  In addition, it saves the bookmark
+state (not the contents) of the bookmarkable buffers in the tab's local
+buffer list."
+  (interactive
+   (list (completing-read
+          "Save bufferlo tab bookmark: "
+          (bufferlo--bookmark-get-names #'bufferlo--bookmark-tab-handler)
+          nil nil nil 'bufferlo-bookmark-tab-history)))
+  (bufferlo--warn)
+  (bookmark-store name (bufferlo--bookmark-tab-get name) no-overwrite)
+  (setf (alist-get 'bufferlo-bookmark-tab-name
+                   (cdr (bufferlo--current-tab)))
+        name)
+  (message "saved bufferlo tab bookmark: %s" name))
+
+(defun bufferlo-bookmark-tab-load (name)
+  "Load a tab bookmark; replace the current tab's state.
+NAME is the bookmark's name."
+  (interactive
+   (list (completing-read
+          "Load bufferlo tab bookmark: "
+          (bufferlo--bookmark-get-names #'bufferlo--bookmark-tab-handler)
+          nil nil nil 'bufferlo-bookmark-tab-history)))
+  (bufferlo--warn)
+  (let ((bookmark-fringe-mark nil))
+    (bookmark-jump name #'ignore))
+  (setf (alist-get 'bufferlo-bookmark-tab-name
+                   (cdr (bufferlo--current-tab)))
+        name))
+
+(defun bufferlo-bookmark-tab-save-current ()
+  "Save the current tab to its associated bookmark.
+The associated bookmark is determined by the name of the bookmark to
+which the tab was last saved or (if not yet saved) from which it was
+initially loaded.  Performs an interactive bookmark selection if no
+associated bookmark exists."
+  (interactive)
+  (bufferlo--warn)
+  (if-let (bm (alist-get 'bufferlo-bookmark-tab-name
+                         (cdr (bufferlo--current-tab))))
+      (bufferlo-bookmark-tab-save bm)
+    (call-interactively #'bufferlo-bookmark-tab-save)))
+
+(defun bufferlo-bookmark-tab-load-current ()
+  "Load the current tab's associated bookmark.
+The associated bookmark is determined by the name of the bookmark to
+which the tab was last saved or (if not yet saved) from which it was
+initially loaded.  Performs an interactive bookmark selection if no
+associated bookmark exists."
+  (interactive)
+  (bufferlo--warn)
+  (if-let (bm (alist-get 'bufferlo-bookmark-tab-name
+                         (cdr (bufferlo--current-tab))))
+      (bufferlo-bookmark-tab-load bm)
+    (call-interactively #'bufferlo-bookmark-tab-load)))
+
+(defun bufferlo-bookmark-frame-save (name &optional no-overwrite)
+  "Save the current frame as a bookmark.
+NAME is the bookmark's name.  If NO-OVERWRITE is non-nil,
+record the new bookmark without throwing away the old one.
+
+This function persists the current frame's state (the \"session\"):
+The resulting bookmark stores the window configurations and the local
+buffer lists of all tabs in the frame.  In addition, it saves the bookmark
+state (not the contents) of the bookmarkable buffers for each tab."
+  (interactive
+   (list (completing-read
+          "Save bufferlo frame bookmark: "
+          (bufferlo--bookmark-get-names #'bufferlo--bookmark-frame-handler)
+          nil nil nil 'bufferlo-bookmark-frame-history
+          (frame-parameter nil 'bufferlo-bookmark-frame-name))))
+  (bufferlo--warn)
+  (bookmark-store name (bufferlo--bookmark-frame-get name) no-overwrite)
+  (set-frame-parameter nil 'bufferlo-bookmark-frame-name name)
+  (message "Saved bufferlo frame bookmark: %s" name))
+
+(defun bufferlo-bookmark-frame-load (name)
+  "Load a frame bookmark; replace the current frame's state.
+NAME is the bookmark's name."
+  (interactive
+   (list (completing-read
+          "Load bufferlo frame bookmark: "
+          (bufferlo--bookmark-get-names #'bufferlo--bookmark-frame-handler)
+          nil nil nil 'bufferlo-bookmark-frame-history
+          (frame-parameter nil 'bufferlo-bookmark-frame-name))))
+  (bufferlo--warn)
+  (let ((bookmark-fringe-mark nil))
+    (bookmark-jump name #'ignore))
+  (set-frame-parameter nil 'bufferlo-bookmark-frame-name name))
+
+(defun bufferlo-bookmark-frame-save-current ()
+  "Save the current frame to its associated bookmark.
+The associated bookmark is determined by the name of the bookmark to
+which the frame was last saved or (if not yet saved) from which it was
+initially loaded.  Performs an interactive bookmark selection if no
+associated bookmark exists."
+  (interactive)
+  (bufferlo--warn)
+  (if-let (bm (frame-parameter nil 'bufferlo-bookmark-frame-name))
+      (bufferlo-bookmark-frame-save bm)
+    (call-interactively #'bufferlo-bookmark-frame-save)))
+
+(defun bufferlo-bookmark-frame-load-current ()
+  "Load the current frame's associated bookmark.
+The associated bookmark is determined by the name of the bookmark to
+which the frame was last saved or (if not yet saved) from which it was
+initially loaded.  Performs an interactive bookmark selection if no
+associated bookmark exists."
+  (interactive)
+  (bufferlo--warn)
+  (if-let (bm (frame-parameter nil 'bufferlo-bookmark-frame-name))
+      (bufferlo-bookmark-frame-load bm)
+    (call-interactively #'bufferlo-bookmark-frame-load)))
+
 (provide 'bufferlo)
 
 ;;; bufferlo.el ends here
